@@ -2,16 +2,27 @@ import yaml
 from absql.functions import default_constructors
 
 
-def scalar_to_value(scalar):
+def scalar_to_value(scalar, constructor_dict):
     """
     Converts a YAML ScalarNode to its underlying Python value
     """
     type = scalar.tag.split(":")[-1]
     val = scalar.value
+    if isinstance(scalar, yaml.MappingNode):
+        val = node_converter(scalar, constructor_dict)
+        func = constructor_dict.get(type)
+        return func(**val)
+    if isinstance(scalar, yaml.SequenceNode):
+        val = node_converter(scalar, constructor_dict)
+        func = constructor_dict.get(type)
+        return func(*val)
+    if type.startswith("!"):
+        func = constructor_dict.get(type)
+        return func(val)
     return eval('{type}("""{val}""")'.format(type=type, val=val))
 
 
-def node_converter(x):
+def node_converter(x, constructor_dict):
     """
     Converts YAML nodes of varying types into Python values,
     lists, and dictionaries
@@ -21,17 +32,22 @@ def node_converter(x):
         return yaml.load(x.value, yaml.SafeLoader)
     if isinstance(x, yaml.SequenceNode):
         # "I am a list"
-        return [scalar_to_value(v) for v in x.value]
+        return [scalar_to_value(v, constructor_dict) for v in x.value]
     if isinstance(x, yaml.MappingNode):
         # "I am a dict"
-        return {scalar_to_value(v[0]): scalar_to_value(v[1]) for v in x.value}
+        return {
+            scalar_to_value(v[0], constructor_dict): scalar_to_value(
+                v[1], constructor_dict
+            )
+            for v in x.value
+        }
 
 
-def wrap_yaml(func):
+def wrap_yaml(func, constructor_dict):
     """Turn a function into one that can be run on a YAML input"""
 
     def ret(loader, x):
-        value = node_converter(x)
+        value = node_converter(x, constructor_dict)
 
         if value is not None:
 
@@ -49,9 +65,10 @@ def wrap_yaml(func):
     return ret
 
 
-def generate_loader(extra_constructors={}):
+def generate_loader(extra_constructors=None):
     """Generates a SafeLoader with both default and custom constructors"""
     loader = yaml.SafeLoader
+    extra_constructors = extra_constructors or []
     default_constructor_dict = default_constructors.copy()
 
     if isinstance(extra_constructors, list) and len(extra_constructors) > 0:
@@ -62,5 +79,5 @@ def generate_loader(extra_constructors={}):
     if len(extra_constructors) > 0:
         default_constructor_dict.update(extra_constructors)
     for tag, func in default_constructor_dict.items():
-        loader.add_constructor(tag, wrap_yaml(func))
+        loader.add_constructor(tag, wrap_yaml(func, default_constructor_dict))
     return loader
