@@ -39,3 +39,16 @@ def test_render_text_only(runner):
     got = runner.render("{{greeting}}, {{env_var('name')}}!")
     want = "Hello, Bob!"
     assert got == want
+
+
+def test_replace_only_changes(runner):
+    original = runner.render("{{env_switch(foo='address')}} and {{greeting}}")
+    assert original == "value_unspecified and Hello"
+
+    replaced_only = runner.render(
+        "{{env_switch(foo='address')}} and {{greeting}}", replace_only=True
+    )
+    assert replaced_only == "{{env_switch(foo='address')}} and Hello"
+
+    original_2 = runner.render("{{env_switch(foo='address')}} and {{greeting}}")
+    assert original_2 == "value_unspecified and Hello"
